@@ -153,7 +153,7 @@ def setup_shared_screen_dir() -> bool:
 
 def setup_passwordless_nexus_attach() -> bool:
     sudoers_file = pl.Path("/etc/sudoers.d/nexus_attach")
-    content = "ALL ALL=(nexus) NOPASSWD: /usr/bin/screen -r *\n"
+    content = "ALL ALL=(nexus) NOPASSWD: /usr/bin/screen -r *, /usr/bin/screen -S * -X acladd *\n"
 
     try:
         sudoers_file.write_text(content)
@@ -163,40 +163,18 @@ def setup_passwordless_nexus_attach() -> bool:
         return False
 
 
-def generate_self_signed_cert(server_dir: pl.Path) -> tuple[pl.Path, pl.Path]:
-    import socket
+def setup_screen_multiuser() -> bool:
+    screen_path = pl.Path("/usr/bin/screen")
+    if not screen_path.exists():
+        return False
 
-    ssl_dir = server_dir / "ssl"
-    ssl_dir.mkdir(parents=True, exist_ok=True)
-
-    key_path = ssl_dir / "key.pem"
-    cert_path = ssl_dir / "cert.pem"
-
-    if key_path.exists() and cert_path.exists():
-        return key_path, cert_path
-
-    subprocess.run(
-        [
-            "openssl",
-            "req",
-            "-x509",
-            "-newkey",
-            "rsa:4096",
-            "-nodes",
-            "-keyout",
-            str(key_path),
-            "-out",
-            str(cert_path),
-            "-days",
-            "3650",
-            "-subj",
-            f"/CN={socket.gethostname()}",
-        ],
-        check=True,
-        capture_output=True,
-    )
-
-    return key_path, cert_path
+    try:
+        current_mode = screen_path.stat().st_mode
+        if not (current_mode & stat.S_ISUID):
+            subprocess.run(["chmod", "u+s", str(screen_path)], check=True)
+        return True
+    except Exception:
+        return False
 
 
 def set_system_permissions() -> None:
@@ -204,10 +182,21 @@ def set_system_permissions() -> None:
     subprocess.run(["chmod", "-R", "770", str(SYSTEM_SERVER_DIR)], check=True)
 
 
+def detect_nexus_server_executable() -> str:
+    exec_path = shutil.which("nexus-server")
+    if not exec_path:
+        sys.exit(
+            "ERROR: nexus-server executable not found in PATH.\n"
+            "Ensure nexusai is installed correctly with: sudo pip3 install nexusai"
+        )
+    return exec_path
+
+
 def setup_systemd_server(sup_groups: list[str] | None = None) -> tuple[bool, str | None]:
     from nexus.server.installation import systemd
 
-    server_content = systemd.get_service_file_content(sup_groups)
+    exec_path = detect_nexus_server_executable()
+    server_content = systemd.get_service_file_content(exec_path, sup_groups)
     dest_server = SYSTEMD_DIR / SYSTEMD_SERVICE_FILENAME
     dest_server.write_text(server_content)
     return True, None
@@ -284,21 +273,6 @@ def setup_config(
 
     base_config = create_interactive_config(default_config) if interactive else default_config
 
-    if base_config.api_token:
-        ssl_dir = server_dir / "ssl"
-        key_path = ssl_dir / "key.pem"
-        cert_path = ssl_dir / "cert.pem"
-        return config.NexusServerConfig(
-            server_dir=base_config.server_dir,
-            port=base_config.port,
-            node_name=base_config.node_name,
-            supplementary_groups=base_config.supplementary_groups,
-            api_token=base_config.api_token,
-            external_ip=base_config.external_ip,
-            ssl_keyfile=str(key_path),
-            ssl_certfile=str(cert_path),
-        )
-
     return base_config
 
 
@@ -307,7 +281,6 @@ def display_config(_config: config.NexusServerConfig) -> None:
     print("======================")
 
     config_dict = _config.model_dump()
-    api_token = config_dict.pop("api_token", None)
 
     for key, value in config_dict.items():
         print(f"{key}: {value}")
@@ -323,10 +296,8 @@ def display_config(_config: config.NexusServerConfig) -> None:
     if external_ip:
         print(f"\nexternal_ip: {external_ip}")
 
-    if api_token:
-        print("\n🔑 API Token:")
-        print(f"   {api_token}")
-        print("   ⚠️  Required for remote CLI connections")
+    print("\n🔒 Authentication: SSH tunnel-based (no API token)")
+    print("   Remote access requires SSH port forwarding")
 
     print()
 
@@ -478,9 +449,13 @@ def setup_screen_run_dir() -> None:
         print(f"Created screen directory at {screen_dir}")
 
     current_perms = stat.S_IMODE(screen_dir.stat().st_mode)
-    if current_perms != 0o777:
-        screen_dir.chmod(0o777)
-        print(f"Set permissions 777 on {screen_dir}")
+    if current_perms != 0o755:
+        screen_dir.chmod(0o755)
+        print(f"Set permissions 755 on {screen_dir}")
+
+        actual_perms = stat.S_IMODE(screen_dir.stat().st_mode)
+        if actual_perms != 0o755:
+            raise RuntimeError(f"Failed to set {screen_dir} permissions to 755 (got {oct(actual_perms)})")
 
 
 def prepare_system_environment(sup_groups: list[str] | None = None) -> None:
@@ -497,6 +472,9 @@ def prepare_system_environment(sup_groups: list[str] | None = None) -> None:
 
     setup_screen_run_dir()
 
+    if setup_screen_multiuser():
+        print("Enabled multiuser screen support (setuid on /usr/bin/screen)")
+
     if setup_passwordless_nexus_attach():
         print(
             "PASSWORDLESS ACCESS ENABLED: Any user can now run 'sudo -u nexus screen -r <session>' without a password"
@@ -513,19 +491,12 @@ def confirm_installation(_config: config.NexusServerConfig) -> bool:
     print(f"   Server directory:  {SYSTEM_SERVER_DIR}")
     print(f"   Config file:       {SYSTEM_SERVER_DIR / 'config.toml'}")
     print(f"   Database:          {SYSTEM_SERVER_DIR / 'nexus_server.db'}")
-    print(f"   SSL certificates:  {SYSTEM_SERVER_DIR / 'ssl/'}")
     print(f"   Systemd service:   {SYSTEMD_DIR / SYSTEMD_SERVICE_FILENAME}")
 
     print("\n⚙️  Configuration:")
     for key, value in _config.model_dump().items():
-        if key == "api_token":
-            continue
         print(f"   {key:20} {value}")
 
-    print("\n🔑 API Token:")
-    print(f"   {_config.api_token}")
-    print("   ⚠️  SAVE THIS - Required for remote CLI connections")
-
     print("\n" + "=" * 80)
     print("Note: Environment variables (NS_*) override config during installation")
     print("=" * 80)
@@ -548,11 +519,11 @@ def print_installation_complete_message(_config: config.NexusServerConfig) -> No
     if _config.external_ip:
         print(f"   Address:      {_config.external_ip}")
     print(f"   Port:         {_config.port}")
-    print(f"   API Token:    {_config.api_token}")
+    print("   Auth:         SSH tunnel (no API token needed)")
 
     print("\n   To connect from CLI, run:")
     print("   nx target add")
-    print("   # Then enter the values above when prompted")
+    print("   # Ensure your SSH key is in ~/.ssh/authorized_keys on this server")
 
     print("\n📝 View Config Anytime:")
     print("   sudo nexus-server config")
@@ -587,10 +558,6 @@ def install_system(
 
     prepare_system_environment(_config.supplementary_groups)
 
-    if _config.api_token:
-        generate_self_signed_cert(SYSTEM_SERVER_DIR)
-        print(f"Generated SSL certificates in: {SYSTEM_SERVER_DIR / 'ssl'}")
-
     create_persistent_directory(_config)
     print(f"Created configuration at: {config.get_config_path(SYSTEM_SERVER_DIR)}")
 
@@ -881,13 +848,23 @@ def _check_screen_permissions() -> None:
     try:
         if not screen_dir.exists():
             screen_dir.mkdir(parents=True, exist_ok=True)
-        if stat.S_IMODE(screen_dir.stat().st_mode) != 0o777:
-            screen_dir.chmod(0o777)
+        if stat.S_IMODE(screen_dir.stat().st_mode) != 0o755:
+            screen_dir.chmod(0o755)
+            actual_perms = stat.S_IMODE(screen_dir.stat().st_mode)
+            if actual_perms != 0o755:
+                raise RuntimeError(f"Failed to set {screen_dir} permissions to 755 (got {oct(actual_perms)})")
     except (OSError, PermissionError) as e:
-        raise RuntimeError(
-            f"Screen requires {screen_dir} with permissions 777. Error: {e}\n"
-            f"Fix: sudo mkdir -p {screen_dir} && sudo chmod 777 {screen_dir}"
-        )
+        try:
+            print(f"Attempting to fix {screen_dir} permissions using sudo...")
+            subprocess.run(["sudo", "mkdir", "-p", str(screen_dir)], check=True, capture_output=True)
+            subprocess.run(["sudo", "chmod", "755", str(screen_dir)], check=True, capture_output=True)
+            print(f"Successfully fixed {screen_dir} permissions.")
+        except (subprocess.CalledProcessError, FileNotFoundError) as sudo_error:
+            raise RuntimeError(
+                f"Screen requires {screen_dir} with permissions 755. Error: {e}\n"
+                f"Attempted automatic fix with sudo but failed: {sudo_error}\n"
+                f"Manual fix: sudo mkdir -p {screen_dir} && sudo chmod 755 {screen_dir}"
+            )
 
 
 def initialize_context(server_dir: pl.Path | None) -> context.NexusServerContext:
